/**
 * 
 */
package net.java.amateras.uml.sequence.model;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.java.amateras.xstream.XStreamSerializer;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * @author Takahiro Shida.
 * 
 */
public class SequenceModelBuilder {

	private InteractionModel root = new InteractionModel();

	private ActivationModel current;

	private Map messageMap = new HashMap();

	private int currentY = 0;
	
	public InstanceModel createInstance(String instanceName) {
		InstanceModel model = new InstanceModel();
		model.setName(instanceName);
		model.setConstraint(new Rectangle(
				120 * root.getInstances().size() + 20,
				InstanceModel.DEFAULT_LOCATION, 100, -1));
		Rectangle lineRect = model.getConstraint().getCopy();
		lineRect.translate(new Point(50, 0));
		lineRect.width = 5;
		lineRect.height = LifeLineModel.DEFAULT_HEIGHT;
		model.getModel().setConstraint(lineRect);
		root.addInstance(model);
		return model;
	}

	public void init(InstanceModel instance) {
		ActivationModel model = new ActivationModel();
		model.setConstraint(new Rectangle(instance.getConstraint().x + 45, 50,
				ActivationModel.DEFAULT_WIDTH, ActivationModel.DEFAULT_HEIGHT));
		instance.getModel().addActivation(model);
		current = model;
		currentY = 70;
	}

	/**
	 *  
	 * @param key [source instance name ]-[message name]-[target instance name]
	 */
	public void back(String key) {
		MessageModel model = (MessageModel) messageMap.get(key);
		back(model);
	}

	/**
	 * 
	 * @param model
	 */
	public void back(MessageModel model) {
		if (model != null) {
			ActivationModel target = (ActivationModel) model.getTarget();
			ActivationModel source = (ActivationModel) model.getSource();
			current = source;
			currentY = target.getConstraint().y + target.getConstraint().height + 20;
		}		
	}
	public MessageModel createMessage(String message, InstanceModel target) {
		ActivationModel model = new ActivationModel();
		ActivationModel targetModel = getTargetModel(
				currentY, target);
		root.addActivation(model);
		if (targetModel == null) {
			model.setConstraint(new Rectangle(target.getConstraint().x + 45,
					currentY,
					ActivationModel.DEFAULT_WIDTH,
					ActivationModel.DEFAULT_HEIGHT));
			target.getModel().addActivation(model);
		} else {
			model.setConstraint(new Rectangle(target.getConstraint().x + 45
					+ current.getNestLevel() * 5,
					current.getConstraint().y + 20,
					ActivationModel.DEFAULT_WIDTH,
					ActivationModel.DEFAULT_HEIGHT));
			targetModel.addActivation(model);
		}
		SyncMessageModel messageModel = new SyncMessageModel();
		messageModel.setName(message);
		messageModel.setSource(current);
		messageModel.setTarget(model);
		messageModel.attachSource();
		messageModel.attachTarget();
		ReturnMessageModel returnMessageModel = new ReturnMessageModel();
		returnMessageModel.setSource(model);
		returnMessageModel.setTarget(current);
		returnMessageModel.attachSource();
		returnMessageModel.attachTarget();
		messageMap.put(current.getOwnerLine().getOwner().getName() + "-"
				+ message + "-" + target.getName(), messageModel);
		model.computeCaller();
		current = model;
		currentY += 20;
		return messageModel;
	}

	private ActivationModel getTargetModel(int y, InstanceModel target) {
		List children = target.getModel().getChildren();
		for (Iterator iter = children.iterator(); iter.hasNext();) {
			ActivationModel element = (ActivationModel) iter.next();
			if (element.getConstraint().y < y
					&& element.getConstraint().y
							+ element.getConstraint().height > y) {
				return element;
			}
		}
		return null;
	}

	public String save() {
		MessageOrderUtil.computeMessageOrders(root);
		return XStreamSerializer.serialize(root, getClass().getClassLoader());
	}
	// public ActivationModel createInstanciateMessage(String message,
	// ActivationModel source, InstanceModel target) {
	//		
	// }
	//
	// public ActivationWrapper createRecursiveMessage(String message,
	// ActivationWrapper source, InstanceWrapper target) {
	//		
	// }

}
